@@ -16,7 +16,6 @@ from copy import deepcopy
 from datetime import datetime
 from typing import no_type_check, Dict, Type, Callable, get_origin, get_args, Set, Any, ForwardRef, List, Tuple
 
-import inflection
 from bson import ObjectId
 
 from .error import SchemaError, DataError, PathError
@@ -337,9 +336,9 @@ class RelationField(ModelField):
                  **kwargs):
         """ Init method.
 
-        :param save_field_name: The name to use for saving current relation
+        :param save_field_name: The name to use for saving current relation, if it is none, will use xxx_id/xxx_ids by default. THe back field in related model have same value, used for search the original model
         :param save_field_order: If the field is List, use this order when loading and saving
-        :param back_field_name: The name to use for the relation from the related object back to this one
+        :param back_field_name: The name to use for the relation from the related model back to this one, if it is none, do not create back field in related model
         :param back_field_is_list: If back field is list
         :param back_field_order: If back field is list, use this order when loading and saving
         :param is_back_field: If its a back field defined in related object
@@ -436,7 +435,7 @@ class ModelMeta(ABCMeta):
                 # BaseModel do not have __fields__
                 if base is not BaseModel:
                     fields.update(deepcopy(base.__fields__))
-                # Id field's type should be defined in BaseModel/MongoModel, so we need to fetch them from bases
+                # Id field's type should be defined in MongoModel/CacheModel, so we need to fetch them from bases
                 if '__id_type__' not in namespace:
                     namespace['__id_type__'] = base.__id_type__
                     namespace['__id_name__'] = base.__id_name__
@@ -507,7 +506,7 @@ class ModelMeta(ABCMeta):
                     skips.add(ann_name)
                     continue
                 # Validate
-                _, f_origin = _validate_annotation(ann_name, ann_type)
+                f_type, f_origin = _validate_annotation(ann_name, ann_type)
                 # Create another field for RelationField
                 save_field = None
                 #
@@ -530,21 +529,24 @@ class ModelMeta(ABCMeta):
                         else:
                             field.save_field_name = ann_name + '_id'
                     # Create the save field
-                    # Please note: if it is none means it is the auto-created back relation in related object
-                    id_type = namespace.get('__id_type__')
+                    id_type = f_type.__id_type__  # related model's is a basemodel
+                    if id_type is None:
+                        raise SchemaError(f'{field.save_field_name}: id type can not be blank')
+                    #
                     save_field = ModelField(
-                        name=field.save_field_name, type_=List[id_type] if f_origin is list else id_type,
+                        name=field.save_field_name,
+                        type_=List[id_type] if f_origin is list else id_type,
                         required=field.required,
                         source_field_name=ann_name,  # Mark the source field
                     )
                     # Set required to false
                     field.required = False
-                    # back_field should always has a meaningful value
-                    # if it is none, this field's object name is used, e.g, user.team -> team.users
+                    # back_field_name should always has a meaningful value, if it is none, do not create back field in related object
+                    # e.g,
+                    # project.activities.user -> user, do not need to create back_field in user model as it is hard to search all the activities for user
+                    # in such case, we need to create activity model, activity.project -> project.activities, activity.user -> user.activities
                     if field.back_field_name is None:
-                        field.back_field_name = name.lower()
-                        if field.back_field_is_list:
-                            field.back_field_name = inflection.pluralize(field.back_field_name)
+                        pass
                 # Define a ModelField
                 elif isinstance(value, ModelField):
                     # Supplement field's name and type, as we use annotation to define a model field
@@ -605,7 +607,10 @@ class ModelMeta(ABCMeta):
             elif isinstance(check_type, SimpleEnumMeta):
                 pass
             elif issubclass(check_type, BaseModel):
-                #
+                # do not iterate check_type.__fields__ but iterate __annotations__ instead
+                # e.g,
+                # if use check_type.__fields__, when checking project.members, will fall into user.team and team.members recursion
+                # because the import mechanism, it is not possible to make explicit reference between user and team, so using annotations can prevent the recursion
                 for a_n, a_t in check_type.__dict__.get('__annotations__', {}).items():
                     _iter(a_n, a_t, level + (1 if is_in_list else 0))
 
@@ -651,7 +656,7 @@ class ModelMeta(ABCMeta):
         # Try to create back field of relation fields after this class is created
         #
         for f in cls.__fields__.values():
-            if isinstance(f, RelationField):
+            if isinstance(f, RelationField) and f.back_field_name:  # Do not create back field in related model if back_field_name is None
                 back_field = RelationField(
                     name=f.back_field_name, type_=List[cls] if f.back_field_is_list else cls,
                     save_field_name=f.save_field_name, save_field_order=f.back_field_order,
@@ -684,8 +689,8 @@ class BaseModel(metaclass=ModelMeta):
     # Fields may be overwrited
     #
     # The id field name
-    __id_name__ = '_id'
-    __id_type__ = ObjectId
+    __id_name__ = None
+    __id_type__ = None
     #
     __icon__ = None
     __title__ = None
@@ -761,19 +766,34 @@ class BaseModel(metaclass=ModelMeta):
                     pass
                 # If it is not Undefined, means it is already loaded and may be updated programmtically, so overwrite current field
                 else:
+                    update_value = None
                     if isinstance(relation_value, list):
+                        relation_type = get_args(source_field.type)[0]
                         update_value = []
                         for v in relation_value:
                             if isinstance(v, dict):  # Value can be raw dict against related model
-                                update_value.append(v.get(cls.__id_name__))
+                                id_ = v.get(relation_type.__id_name__)
                             else:  # Value should be instance of relatited model
-                                update_value.append(getattr(v, cls.__id_name__))
+                                id_ = getattr(v, relation_type.__id_name__)
+                            #
+                            if id_:
+                                if not isinstance(id_, relation_type.__id_type__):
+                                    id_ = relation_type.__id_type__(id_)
+                                #
+                                update_value.append(id_)
                     else:
+                        relation_type = source_field.type
                         if isinstance(relation_value, dict):
-                            update_value = relation_value.get(cls.__id_name__)
+                            id_ = relation_value.get(relation_type.__id_name__)
                         else:
-                            update_value = getattr(relation_value, cls.__id_name__)
-                    #
+                            id_ = getattr(relation_value, relation_type.__id_name__)
+                        #
+                        if id_:
+                            if not isinstance(id_, relation_type.__id_type__):
+                                id_ = relation_type.__id_type__(id_)
+                            #
+                            update_value = id_
+                    # update_value can be none or [], meaning clear the field
                     data[field_name] = update_value
             #
             field_value, field_errors = cls._validate_field(field_type, data.get(field_name, Undefined))
@@ -907,22 +927,22 @@ class BaseModel(metaclass=ModelMeta):
                 if f_origin is None:
                     # This is a back field created by relation field, means id is saved in one relation object
                     if field.is_back_field:
-                        default = f_type.find_one({field.save_field_name: self.__dict__.get(f_type.__id_name__)})
+                        default = f_type.find_one({field.save_field_name: self.__dict__.get(self.__class__.__id_name__)})
                     else:
                         default = f_type.find_one({f_type.__id_name__: self.__dict__.get(field.save_field_name)})
                 elif f_origin is list:
                     l_type = get_args(f_type)[0]
-                    # This is a back field created by relation field, means this object id is saved in many relation objects
+                    # This is a back field created by relation field, means this object id is saved in many related objects
                     if field.is_back_field:
+                        # TODO: support paging, e.g, team.members__2 to fetch page 2's records and defined a back_field_page_size to config paging size
                         # Typical many-to-one definition way, i.e, using a foreign key to store related object's id
                         # In such case, may return very big amount of object so we only fetch part of it, i.e, 100 records
                         # e.g,
-                        # Relation field is defined in User using field name team, so the team id stores in a field name team_id
-                        # Below back relation field auto-created in Team
-                        #   members: List[User] -> User.find({team_id, self._id}, sort=[{team.join_time, 1}])
+                        # Relation field is defined in User using field name team, so the team id stores in a field name team_id, a back relation field with name members is auto-created in Team
+                        # user.team -> team.members, then team.members = User.find({team_id: self._id}, sort=[{team.join_time, 1}])
                         default = list(
                             l_type.find(
-                                {field.save_field_name: self.__dict__.get(l_type.__id_name__)},
+                                {field.save_field_name: self.__dict__.get(self.__class__.__id_name__)},
                                 sort=field.save_field_order,
                                 limit=100)
                         )
@@ -931,7 +951,7 @@ class BaseModel(metaclass=ModelMeta):
                         # In such case, the ids field should not be very large, so we fetch all the objects and sort them by id's position
                         # e.g,
                         # Relation field is defined in Team using field name members, so the team ids store in a field name members_ids
-                        #   members: List[User] -> User.find({_id: {$in: self.members_ids}})
+                        # team.members -> user.team, then team.members = User.find({_id: {$in: self.members_ids}})
                         ids = self.__dict__.get(field.save_field_name)
                         if ids is None:
                             default = []
@@ -1095,6 +1115,16 @@ class BaseModel(metaclass=ModelMeta):
         """ Convert to json str. """
         return json.dumps(self.dict(), cls=ModelJSONEncoder, **kwargs)
 
+    @classmethod
+    def find_one(cls, filter_or_id, *args, **kwargs):
+        """ abstract method, use it to fetch one related model. """
+        raise NotImplementedError()
+
+    @classmethod
+    def find(cls, *args, **kwargs):
+        """ abstract method, use it to fetch many related models. """
+        raise NotImplementedError()
+
     @classmethod
     def schema(cls):
         """ To json schema dict.
@@ -1213,6 +1243,7 @@ class BaseModel(metaclass=ModelMeta):
                     # relation
                     if isinstance(f_t, RelationField):
                         field_schema.update({'is_relation': True})
+                        field_schema.update({'is_back_relation': f_t.is_back_field})
                         relations.append(f_n)
                     #
                     properties[f_n] = field_schema
@@ -1243,7 +1274,8 @@ class BaseModel(metaclass=ModelMeta):
                 obj['sortables'] = sortables
                 # relations
                 obj['relations'] = relations
-                #
+                # id_name
+                obj['id_name'] = type_.__id_name__
                 return obj
             elif type_ is str:
                 return {'type': 'string', 'format': Format.TEXT, 'py_type': 'str'}
