@@ -8,19 +8,16 @@
     :copyright: (c) 2019 by weiminfeng.
     :date: 2019/12/15
 """
-import math
-import re
 from copy import deepcopy
-from datetime import datetime
 
-import inflection
 import pymongo
 from bson import ObjectId
 from pymongo import MongoClient, uri_parser, ReadPreference, WriteConcern
 from pymongo.cursor import Cursor as PyMongoCursor
 
 from .error import DatabaseError, DataError
-from .model import BaseModel, Comparator, SimpleEnumMeta
+from .model import BaseModel
+from .utils import Pagination
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Connection - Support multiple database
@@ -35,9 +32,6 @@ _connections = {}
 # {alias:database of pymongo.Database}
 _dbs = {}
 
-# Valid datetime formats
-_valid_formats = ['%Y-%m-%d %H:%M:%S', '%Y-%m-%d %H:%M:%S.%f', '%Y-%m-%d']
-
 
 def connect(uri, alias=DEFAULT_CONNECTION_NAME, **kwargs):
     """ Connect to database by uri, database name will be extracted from the uri and used for different MongoClient. """
@@ -170,6 +164,10 @@ class MongoModel(BaseModel):
     # if None DEFAULT_CONNECTION_NAME
     __db_alias__ = None
 
+    # id field definition
+    __id_name__ = '_id'
+    __id_type__ = ObjectId
+
     # Mongo ID field
     _id: ObjectId = None
 
@@ -275,15 +273,15 @@ class MongoModel(BaseModel):
         return ModelCursor(cls, collection, *args, **kwargs)
 
     @classmethod
-    def search(cls, filter=None, page=1, per_page=20, max_page=-1, **kwargs):
+    def search(cls, filter_=None, page=1, per_page=20, max_page=-1, **kwargs):
         """ Search models and return records and pagination. """
-        count = cls.count(filter)
+        count = cls.count(filter_)
         if max_page > 0:
             limit = per_page * max_page
             if count > limit:
                 count = limit
         start = (page - 1) * per_page
-        records = list(cls.find(filter, skip=start, limit=per_page, **kwargs))
+        records = list(cls.find(filter_, skip=start, limit=per_page, **kwargs))
         pagination = Pagination(page, per_page, count)
         return records, pagination
 
@@ -318,45 +316,46 @@ class MongoModel(BaseModel):
             return None
 
     @classmethod
-    def count(cls, filter=None, **kwargs):
+    def count(cls, filter_=None, **kwargs):
+        """ Count reconds. """
         collection = cls.get_collection(**kwargs)
-        if filter:
-            return collection.count_documents(filter, **kwargs)
+        if filter_:
+            return collection.count_documents(filter_, **kwargs)
         else:
             return collection.estimated_document_count(**kwargs)
 
     @classmethod
-    def replace_one(cls, filter, replacement, *args, **kwargs):
+    def replace_one(cls, filter_, replacement, *args, **kwargs):
         """ Please note we do not apply validation here. """
         collection = cls.get_collection(**kwargs)
         # UpdateResult
-        return collection.replace_one(filter, replacement, *args, **kwargs)
+        return collection.replace_one(filter_, replacement, *args, **kwargs)
 
     @classmethod
-    def update_one(cls, filter, update, *args, **kwargs):
+    def update_one(cls, filter_, update, *args, **kwargs):
         """ Please note we do not apply validation here. """
         collection = cls.get_collection(**kwargs)
         # UpdateResult
-        return collection.update_one(filter, update, *args, **kwargs)
+        return collection.update_one(filter_, update, *args, **kwargs)
 
     @classmethod
-    def update_many(cls, filter, update, *args, **kwargs):
+    def update_many(cls, filter_, update, *args, **kwargs):
         """ Please note we do not apply validation here. """
         collection = cls.get_collection(**kwargs)
         # UpdateResult
-        return collection.update_many(filter, update, *args, **kwargs)
+        return collection.update_many(filter_, update, *args, **kwargs)
 
     @classmethod
-    def delete_one(cls, filter, **kwargs):
+    def delete_one(cls, filter_, **kwargs):
         collection = cls.get_collection(**kwargs)
         # DeleteResult
-        return collection.delete_one(filter)
+        return collection.delete_one(filter_)
 
     @classmethod
-    def delete_many(cls, filter, **kwargs):
+    def delete_many(cls, filter_, **kwargs):
         collection = cls.get_collection(**kwargs)
         # DeleteResult
-        return collection.delete_many(filter)
+        return collection.delete_many(filter_)
 
     @classmethod
     def aggregate(cls, pipeline, **kwargs):
@@ -368,9 +367,9 @@ class MongoModel(BaseModel):
         return collection.aggregate(pipeline, **kwargs)
 
     @classmethod
-    def distinct(cls, key, filter=None, **kwargs):
+    def distinct(cls, key, filter_=None, **kwargs):
         collection = cls.get_collection(**kwargs)
-        return collection.distinct(key, filter, **kwargs)
+        return collection.distinct(key, filter_, **kwargs)
 
     @classmethod
     def group(cls, key, condition, initial, reduce, finalize=None, **kwargs):
@@ -425,271 +424,3 @@ class ModelCursor(PyMongoCursor):
             return super(ModelCursor, self).__getitem__(index)
         else:
             return self.model_class(super(ModelCursor, self).__getitem__(index))
-
-
-# ----------------------------------------------------------------------------------------------------------------------
-# Pagination
-#
-
-class Pagination(object):
-    """ Pagination support. """
-
-    def __init__(self, page, per_page, total_count):
-        self.page = page
-        self.per_page = per_page
-        self.total_count = total_count
-
-    @property
-    def pages(self):
-        return int(math.ceil(self.total_count / float(self.per_page)))
-
-    @property
-    def has_prev(self):
-        return self.page > 1
-
-    @property
-    def prev(self):
-        return self.page - 1 if self.has_prev else None
-
-    @property
-    def has_next(self):
-        return self.page < self.pages
-
-    @property
-    def next(self):
-        return self.page + 1 if self.has_next else None
-
-    @property
-    def iter_pages(self, left_edge=2, left_current=2, right_current=3, right_edge=2):
-        last = 0
-        for num in range(1, self.pages + 1):
-            if num <= left_edge or \
-                    (num > self.page - left_current - 1 and num < self.page + right_current) or \
-                    num > self.pages - right_edge:
-                if last + 1 != num:
-                    yield None
-                yield num
-                last = num
-
-    def __iter__(self):
-        """ Can use dict(pagination) for jsonify. """
-        for key in ['page', 'pages', 'prev', 'next']:
-            yield key, getattr(self, key)
-        #
-        yield 'iter_pages', list(self.iter_pages)
-
-
-# ----------------------------------------------------------------------------------------------------------------------
-# Html request processing
-#
-
-def _multidict_decode(md, dict_char='.', list_char='-'):
-    """ Decode a multi-dict into a nested dict. """
-    result = {}
-    dicts_to_sort = set()
-    for key, value in md.items():
-        # Split keys into tokens by dict_char and list_char
-        keys = _normalized_path(key).split(dict_char)
-        new_keys = []
-        for k in keys:
-            if list_char in k:
-                list_tokens = k.split(list_char)
-                # For list tokens, the 1st one should always be field name, the latter ones are indexes
-                for i in range(len(list_tokens)):
-                    if list_tokens[i].isdigit():
-                        new_keys.append(int(list_tokens[i]))
-                    else:
-                        new_keys.append(list_tokens[i])
-                    if i < len(list_tokens) - 1:
-                        dicts_to_sort.add(tuple(new_keys))
-            else:
-                new_keys.append(k)
-
-        # Create inner dicts, lists are also initialized as dicts
-        place = result
-        for i in range(len(new_keys) - 1):
-            try:
-                if not isinstance(place[new_keys[i]], dict):
-                    place[new_keys[i]] = {None: place[new_keys[i]]}
-                place = place[new_keys[i]]
-            except KeyError:
-                place[new_keys[i]] = {}
-                place = place[new_keys[i]]
-
-        # Fill the contents
-        if new_keys[-1] in place:
-            if isinstance(place[new_keys[-1]], dict):
-                place[new_keys[-1]][None] = value
-            elif isinstance(place[new_keys[-1]], list):
-                if isinstance(value, list):
-                    place[new_keys[-1]].extend(value)
-                else:
-                    place[new_keys[-1]].append(value)
-            else:
-                if isinstance(value, list):
-                    place[new_keys[-1]] = [place[new_keys[-1]]]
-                    place[new_keys[-1]].extend(value)
-                else:
-                    place[new_keys[-1]] = [place[new_keys[-1]], value]
-        else:
-            place[new_keys[-1]] = value
-
-    # Convert sorted dict to list
-    to_sort_list = sorted(dicts_to_sort, key=len, reverse=True)
-    for key in to_sort_list:
-        to_sort = result
-        source = None
-        last_key = None
-        for sub_key in key:
-            source = to_sort
-            last_key = sub_key
-            to_sort = to_sort[sub_key]
-        if None in to_sort:
-            none_values = [(0, x) for x in to_sort.pop(None)]
-            none_values.extend(iter(to_sort.items()))
-            to_sort = none_values
-        else:
-            to_sort = iter(to_sort.items())
-
-        to_sort = [x[1] for x in sorted(to_sort, key=_sort_key)]
-        source[last_key] = to_sort
-
-    return result
-
-
-def _sort_key(item):
-    """ Robust sort key that sorts items with invalid keys last.
-    This is used to make sorting behave the same across Python 2 and 3.
-    """
-    key = item[0]
-    return not isinstance(key, int), key
-
-
-def populate_model(multidict, model_cls, set_default=True):
-    """ Create a model instance from a multidict.
-    This is necessary because some HTML form elements pass multiple values for the same key.
-
-    :param multidict: multiple values for the same key, e.g, MultiDict([('a', 'b'), ('a', 'c')])
-    """
-    d = {}
-    model_prefix = model_cls.__name__.lower() + '.'  # demouser.name
-    model_prefix_underscore = inflection.underscore(model_cls.__name__) + '.'  # demo_user.name
-    for key, value in multidict.items():
-        # NOTE: Blank string skipped
-        if not value:
-            continue
-        # Only process the keys with leading model.
-        if key.startswith(model_prefix):
-            key = key[len(model_prefix):]
-        elif key.startswith(model_prefix_underscore):
-            key = key[len(model_prefix_underscore):]
-        else:
-            continue
-        #
-        t = model_cls.get_type(key)
-        if isinstance(value, list):
-            converted_value = [convert_from_string(v, t) for v in value if v]
-        else:
-            converted_value = convert_from_string(value, t)
-        #
-        d[key] = converted_value
-    #
-    d = _multidict_decode(d)
-    return model_cls(d)
-
-
-def populate_search(multidict, model_cls):
-    """ Create a condition from search query.
-
-    :returns: search - return to page, condition - send to pymongo for search
-    """
-    search, condition = {}, {}
-    for k, v in multidict.items():
-        if not k.startswith('search.') or not v:
-            continue
-        # Remove search. from k
-        k = k.replace('search.', '')
-        v = v.strip()
-        search[k] = v
-        # Set default comparator
-        c = Comparator.EQ
-        if '__' in k:
-            k, c = k.split('__')
-        #
-        t = model_cls.get_type(k)
-        if Comparator.EQ == c:
-            cond = convert_from_string(v, t)
-        elif Comparator.IN == c or Comparator.NIN == c:
-            cond = {'$%s' % c: [convert_from_string(vv, t) for vv in v]}
-        elif Comparator.LIKE == c:
-            # In order to use index, we only support starting string search here
-            # https://docs.mongodb.com/manual/reference/operator/query/regex/#index-use
-            regx = re.compile('^%s' % re.escape(v))
-            cond = {'$regex': regx}
-        else:
-            cond = {'$%s' % c: convert_from_string(v, t)}
-        #
-        if k not in condition:
-            condition[k] = cond
-        else:
-            condition[k].update(cond)
-    #
-    return search, condition
-
-
-def _normalized_path(path, list_char='-'):
-    """ Change [] -> - for easier processing.
-
-    e.g, user.roles[0] -> user.roles-0
-    """
-    return path.replace('[', list_char).replace(']', '')
-
-
-class DefaultTypeConverter(object):
-    """ Convert string values from html request into typed. """
-
-    def _convert_from_string(self, string_value, type):
-        try:
-            return type(string_value)
-        except ValueError:
-            raise ValueError("can not convert %s to %s" % (string_value, type.__name__))
-
-
-class BoolConverter(DefaultTypeConverter):
-    """ str -> bool """
-
-    def _convert_from_string(self, string_value, type):
-        return string_value.strip().lower() in ("yes", "true")
-
-
-class DatetimeConverter(DefaultTypeConverter):
-    """ str -> datetime """
-
-    def _convert_from_string(self, string_value, type):
-        for fmt in _valid_formats:
-            try:
-                return datetime.strptime(string_value, fmt)
-            except ValueError:
-                pass
-        raise ValueError("can not convert %s to %s" % (string_value, type.__name__))
-
-
-type_converters = {
-    bool: BoolConverter(),
-    datetime: DatetimeConverter(),
-    None: DefaultTypeConverter(),  # Default converter
-}
-
-
-def convert_from_string(string_value, t):
-    if isinstance(t, SimpleEnumMeta):
-        t = t.type
-    if isinstance(string_value, t):
-        return string_value
-    #
-    if t in type_converters:
-        converter = type_converters[t]
-    else:
-        converter = type_converters[None]
-    #
-    return converter._convert_from_string(string_value, t)
